/*
ID: espr1t
TASK: Graph
KEYWORDS: NP, Graph, Weighted MST
CONTEST: CodeIT 2013-2014, Round 1
*/

#include <cstdio>
#include <ctime>
#include <cstring>
#include <cassert>
#include <vector>
#include <algorithm>

using namespace std;
 
const int MAX_NODES = 302;
const double MAX_EXEC_TIME = 1.7;
const long long INF = 1000000000000000000LL;
const int PRINT_DEBUG = 0;


struct Edge {
    int node1, node2, price, idx;
    Edge(int node1_ = 0, int node2_ = 0, int price_ = 0, int idx_ = 0) :
        node1(node1_), node2(node2_), price(price_), idx(idx_) {};
    bool operator < (const Edge& r) const {
        return price != r.price ? price < r.price : idx < r.idx;
    }
};

long long best;
vector <int> answer;

int numNodes, numEdges;
vector <Edge> edges;
vector <int> v[MAX_NODES];
int length[MAX_NODES][MAX_NODES];
long long weight[MAX_NODES][MAX_NODES];


unsigned startTime;
double execTime = 0.0;
inline double getTime() {
    return execTime = (double)(clock() - startTime) / CLOCKS_PER_SEC;
}


int origin;
vector <int> w[MAX_NODES];
int prc[MAX_NODES][MAX_NODES];

long long getPrice(int node, int prev, int price) {
    prc[origin][node] = price;
    long long ret = weight[origin][node] * price;
    for (int i = 0; i < (int)w[node].size(); i++) if (w[node][i] != prev)
        ret += getPrice(w[node][i], node, price + length[node][w[node][i]]);
    return ret;
}

long long checkAnswer(const vector<int>& e, bool printResult) {
    assert((int)e.size() == numNodes - 1);

    long long ans = 0;
    for (int i = 0; i < numNodes; i++)
        w[i].clear();
    for (int i = 0; i < (int)e.size(); i++) {
        w[edges[e[i]].node1].push_back(edges[e[i]].node2);
        w[edges[e[i]].node2].push_back(edges[e[i]].node1);
    }
    for (origin = 0; origin < numNodes; origin++)
        ans += getPrice(origin, -1, 0);
    ans /= 2;
    if (ans < best)
        best = ans, answer = e;
    if (PRINT_DEBUG && printResult)
        fprintf(stderr, "  >> found answer: %lld\n", ans);
    return ans;
}


int parent[MAX_NODES];
int getParent(int node) {
    return parent[node] == node ? node : parent[node] = getParent(parent[node]);
}


vector <int> mst() {
    vector <int> edgeIdx;
    vector <Edge> e = edges;
    sort(e.begin(), e.end());
    for (int node = 0; node < numNodes; node++)
        parent[node] = node;
    for (int i = 0; i < (int)e.size(); i++) {
        if (getParent(e[i].node1) != getParent(e[i].node2)) {
            parent[getParent(e[i].node1)] = getParent(e[i].node2);
            edgeIdx.push_back(e[i].idx);
        }
    }
    return edgeIdx;
}


vector <int> greedy() {
    vector <int> edgeIdx, nodeSet;
    vector <bool> inSet(numNodes, false);
    vector < vector <int> > dist(numNodes, vector <int>(numNodes, 0));
    
    vector <long long> sum(numNodes, 0);
    vector < vector <long long> > pre(numNodes, vector <long long>(numNodes, 0));

    int root = 0;
    nodeSet.push_back(root); inSet[root] = true;
    for (int node = 0; node < numNodes; node++) if (!inSet[node]) {
        sum[node] += weight[node][root];
        pre[node][root] += sum[node] * length[node][root];
    }

    long long prc = 0;
    for (int edge = 0; edge < numNodes - 1; edge++) {
        long long best = INF, with = -1;
        for (int i = 0; i < (int)edges.size(); i++) {
            if (inSet[edges[i].node1] ^ inSet[edges[i].node2]) {
                if (inSet[edges[i].node1])
                    swap(edges[i].node1, edges[i].node2);
                if (best > pre[edges[i].node1][edges[i].node2])
                    best = pre[edges[i].node1][edges[i].node2], with = i;
            }
        }
        // Check that we actually found an edge (should always be true)
        assert(with != -1);
        int nNode = edges[with].node1, sNode = edges[with].node2;
        
        // Add new node and edge to the tree
        edgeIdx.push_back(edges[with].idx);
        nodeSet.push_back(nNode); inSet[nNode] = true;
        
        // Debug
        prc += pre[nNode][sNode];

        // Update inner distances
        for (int i = 0; i < (int)nodeSet.size() - 1; i++)
            dist[nNode][nodeSet[i]] = dist[nodeSet[i]][nNode] =
                edges[with].price + dist[sNode][nodeSet[i]];

        // Initialize new entry point and update old ones
        for (int node = 0; node < numNodes; node++) if (!inSet[node]) {
            sum[node] += weight[node][nNode];
            pre[node][nNode] = sum[node] * length[node][nNode];
            for (int i = 0; i < (int)nodeSet.size() - 1; i++) {
                pre[node][nNode] += weight[node][nodeSet[i]] * dist[nNode][nodeSet[i]];
                pre[node][nodeSet[i]] += weight[node][nNode] * (length[node][nodeSet[i]] + dist[nNode][nodeSet[i]]);
            }
        }
    }
    return edgeIdx;
}


vector <int> optimize(vector <int> use, double timeLimit) {
    while (getTime() < timeLimit) {
        bool found = false;
        random_shuffle(use.begin(), use.end());
        
        long long cp = checkAnswer(use, false);
        for (int edge = 0; edge < (int)use.size(); edge++) {
            for (int node = 0; node < numNodes; node++)
                parent[node] = node;
            for (int i = 0; i < (int)use.size(); i++) if (i != edge)
                parent[getParent(edges[use[i]].node1)] = getParent(edges[use[i]].node2);
            for (int node = 0; node < numNodes; node++)
                parent[node] = getParent(node);
            
            long long price = cp, flow = 0;
            vector <int> g1, g2;
            for (int node = 0; node < numNodes; node++)
                if (parent[node] == parent[0]) g1.push_back(node); else g2.push_back(node);
            
            vector <long long> sum(numNodes, 0);
            for (int i = 0; i < (int)g1.size(); i++) {
                for (int c = 0; c < (int)g2.size(); c++) {
                    flow += weight[g1[i]][g2[c]];
                    sum[g1[i]] += weight[g1[i]][g2[c]];
                    sum[g2[c]] += weight[g1[i]][g2[c]];
                    price -= weight[g1[i]][g2[c]] * prc[g1[i]][g2[c]];
                }
            }
            vector <long long> connect(numNodes, 0);
            for (int i = 0; i < (int)g1.size(); i++)
                for (int c = 0; c < (int)g1.size(); c++)
                    connect[g1[i]] += prc[g1[i]][g1[c]] * sum[g1[c]];
            for (int i = 0; i < (int)g2.size(); i++)
                for (int c = 0; c < (int)g2.size(); c++)
                    connect[g2[i]] += prc[g2[i]][g2[c]] * sum[g2[c]];
            
            for (int i = 0; i < (int)edges.size(); i++) {
                if (parent[edges[i].node1] != parent[edges[i].node2]) {
                    long long cprice = price + edges[i].price * flow;
                    cprice += connect[edges[i].node1] + connect[edges[i].node2];
                    if (cprice < cp) {
                        use[edge] = i;
                        found = true;
                        break;
                    }
                }
            }
            if (found) break;
        }
        if (!found) break;
    }
    return use;
}

void randoms() {
    while (getTime() < MAX_EXEC_TIME) {
        vector <int> idx;
        for (int i = 0; i < (int)edges.size(); i++)
            idx.push_back(i);
        random_shuffle(idx.begin(), idx.end());
        
        for (int node = 0; node < numNodes; node++)
            parent[node] = node;

        vector <int> edgeIdx;
        for (int i = 0; i < (int)idx.size(); i++) {
            int node1 = edges[idx[i]].node1;
            int node2 = edges[idx[i]].node2;
            if (getParent(node1) != getParent(node2)) {
                parent[getParent(node1)] = getParent(node2);
                edgeIdx.push_back(idx[i]);
            }
        }
        edgeIdx = optimize(edgeIdx, MAX_EXEC_TIME);
        checkAnswer(edgeIdx, true);
    }
}


void readInput(const char* fileName) {
    FILE* in = fopen(fileName, "rt");
    fscanf(in, "%d %d", &numNodes, &numEdges);
    for (int edge = 0; edge < numEdges; edge++) {
        int node1, node2, price;
        fscanf(in, "%d %d %d", &node1, &node2, &price);
        node1--; node2--;
        edges.push_back(Edge(node1, node2, price, edge));
        length[node1][node2] = price, v[node1].push_back(node2);
        length[node2][node1] = price, v[node2].push_back(node1);
    }
    for (int node1 = 0; node1 < numNodes; node1++) {
        for (int node2 = 0; node2 < numNodes; node2++) {
            int w;
            fscanf(in, "%d", &w);
            weight[node1][node2] = w;
        }
    }
    fclose(in);

    // Check input for correctness?
    for (int node1 = 0; node1 < numNodes; node1++) {
        if (weight[node1][node1] != 0)
            fprintf(stderr, "weight[node1][node1] != 0\n");
        if (length[node1][node1] != 0)
            fprintf(stderr, "length[node1][node1] != 0\n");
        for (int node2 = 0; node2 < numNodes; node2++) if (node1 != node2) {
            if (weight[node1][node2] != weight[node2][node1])
                fprintf(stderr, "weight[node1][node2] != weight[node2][node1]\n");
            if (length[node1][node2] != length[node2][node1])
                fprintf(stderr, "length[node1][node2] != length[node2][node1]\n");
        }
    }
}


void printOutput(const char* fileName) {
    FILE* out = fopen(fileName, "wt");
    for (int i = 0; i < (int)answer.size(); i++)
        fprintf(out, "%d\n", answer[i] + 1);
    fclose(out);
}


void init() {
    srand(42);
    startTime = clock();

    edges.clear();
    memset(length, 0, sizeof(length));
    memset(weight, 0, sizeof(weight));
    
    best = INF;
    answer.clear();
}


int main(void) {
    init();
    readInput("graph.in");
    
    vector <int> candidate1 = mst();
    checkAnswer(candidate1, true);
    if (PRINT_DEBUG)
        fprintf(stderr, "%.3lfs\n", getTime());
    
    vector <int> candidate2 = greedy();
    checkAnswer(candidate2, true);
    if (PRINT_DEBUG)
        fprintf(stderr, "%.3lfs\n", getTime());
    
    double curTime = getTime();
    double timePerRun = (MAX_EXEC_TIME - curTime) / 3.0;
    candidate1 = optimize(candidate1, curTime + 1.0 * timePerRun);
    checkAnswer(candidate1, true);
    if (PRINT_DEBUG)
        fprintf(stderr, "%.3lfs\n", getTime());
    candidate2 = optimize(candidate2, curTime + 2.0 * timePerRun);
    checkAnswer(candidate2, true);
    if (PRINT_DEBUG)
        fprintf(stderr, "%.3lfs\n", getTime());
    
    if (candidate1 != answer)
        swap(candidate1, candidate2);
    
    candidate1 = optimize(candidate1, MAX_EXEC_TIME);
    candidate2 = optimize(candidate2, MAX_EXEC_TIME);
    checkAnswer(answer, true);
    if (PRINT_DEBUG)
        fprintf(stderr, "%.3lfs\n", getTime());
    
    randoms();
    if (PRINT_DEBUG)
        fprintf(stderr, "%.3lfs\n", getTime());
    
    printOutput("graph.out");
    return 0;
}
